#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# POTRACE - transform bitmaps into vector graphics http://potrace.sourceforge.net/
# 
# Ver 0.1 2019 Initial version.
# Ver 0.2 2019 - resignation from the selection of directories in order to eliminate the problem of spaces in names (which potrace does not tolerate),
#		       - add options "Path visible? ",
#              - the rendered image(layer and path) is named as the source layer (accepts spaces - because the name is given after the autotrace has finished)
# Ver 0.3 2019 - add options "Opttolerance ", remove line print "File size: "
# Ver 0.4 2019 - add options: Copy svg file to another directory, correction color code for Linux.
# Ver 0.5 2020 - separate entry for the menu, internalization - add plugin-trace.po

from gimpfu import *
import subprocess
import sys, os, gettext
from shutil import copyfile
import gettext

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def sys_file(f_name):
	sys_encoded = sys.getfilesystemencoding()
	if os.name == 'nt': coded = f_name.encode(sys_encoded)
	else: coded = f_name
	return(coded)
  
whoiam = os.path.abspath(sys.argv[0])
# initialize internationalisation in a user-locale (by rob_brz1)
locale_directory = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "plugin-potrace", locale_directory, unicode=True )

# --color #rrggbb        - set foreground color (default black #000000)  
def color(colorParam):
	if os.name == 'nt':
		params = " --color #" + str(colorParam)
	else:
		params = " --color '#" + str(colorParam) + "'"
	return params

# --resolution 
def resolutionArg(resolutionParam):
	params = " -r" + str(resolutionParam)
	return params
 
#  --turdsize
def turdsizeArg(turdsizeParam):
	params = " -t" + str(turdsizeParam)
	return params
  
def alphamaxArg(alphamaxParam):
	params = " -a" + " " + str(alphamaxParam) + " -s"
	return params
	
def opttoleranceArg(opttoleranceParam):
	params = " -O " + str(opttoleranceParam)
	return params	
  
def python_fu_potrace(image, layer, saveas, colorParam, addback, backParam, resolutionParam, turdsizeParam, alphamaxParam, opttoleranceParam, pathParam, vis, rem_bmp, copy_svg, outputFolder):

	if saveas==0:
		tempfilename = pdb.gimp_temp_name("bmp")
	if saveas==1:
		tempfilename = pdb.gimp_temp_name("ppm")  

	pdb.gimp_image_set_active_layer(image, layer)
	oldname = layer.name
	tempsvgfilename = pdb.gimp_temp_name("svg")

	tempimage = pdb.gimp_image_duplicate(image)
	if not tempimage:
		raise RuntimeError

	tempdrawable = pdb.gimp_image_get_active_drawable(image)

	pdb.gimp_progress_set_text ("Saving a copy")
	layer.name = layer.name.decode("utf-8")

	if saveas==0:
		xchange_utf = tempfilename
		gimp.pdb.file_bmp_save(tempimage, tempdrawable, xchange_utf, xchange_utf, run_mode=RUN_NONINTERACTIVE)

	if saveas==1:
		pdb.gimp_layer_flatten(tempdrawable)
		xchange_utf = tempfilename  
		pdb.file_ppm_save(tempimage, tempdrawable, xchange_utf, xchange_utf, 0)

	try:
		outfile = open(tempsvgfilename, 'w')
	except:
		raise RuntimeError, "Could not open temporary svg file: " + tempsvgfilename
		
# In Linux binDir = "/usr/bin/potrace" or "/usr/local/bin/potrace" ???   
	if os.name == 'nt':
		binDir="potrace.exe"
	else:
		binDir="/usr/bin/potrace"

	command = binDir \
	+ color(colorParam) \
	+ resolutionArg(resolutionParam) \
	+ turdsizeArg(turdsizeParam) \
	+ alphamaxArg(alphamaxParam) \
	+ opttoleranceArg(opttoleranceParam) \
	+ " " + "--output" \
	+ " " + tempsvgfilename \
	+ " " + xchange_utf

	print command, tempsvgfilename

	pdb.gimp_progress_set_text ("Tracing")
	pdb.gimp_progress_pulse()

	child = subprocess.Popen(command, shell=True, stdout=outfile)
	child.communicate() # wait for child to terminate

	if os.path.getsize(tempsvgfilename) > 600100 and pathParam == 1:  
		# Magic number, wag for file size as of 2009, my computer, etc.
		pdb.gimp_message("Trace plugin error: SVG file has too many individual paths to import.")
		return  # Leaves files in gimp/tmp, to get cleaned later
    
	resolutionx, resolutiony = pdb.gimp_image_get_resolution(tempimage) # Same resolution as original
	pdb.gimp_progress_set_text ("Rendering")
  
	try:
		if pathParam==3:
			svgimage = pdb.file_svg_load(tempsvgfilename, "", layer.width, layer.height, 0, 1, run_mode=RUN_INTERACTIVE)
		else:  
			svgimage = pdb.file_svg_load(tempsvgfilename, "", resolutionx, layer.width, layer.height, pathParam)	
	except RuntimeError:
		pdb.gimp_message("The underlying Potrace program failed for unknown reasons, possibly not enough resources.Try lower quality.")

	#vectors = pdb.gimp_image_get_vectors(svgimage)
	#print "File size: ", os.path.getsize(tempsvgfilename)," Paths: ", len(vectors[1])
	
	active_layer = pdb.gimp_image_get_active_layer(svgimage)
	active_layer.name = active_layer.name.replace("Rendered SVG", oldname)
	active_layer.name = active_layer.name.replace("Wyrenderowany plik SVG", oldname)	
	
	if addback == 1:
		newLayer = gimp.Layer(svgimage, "Background", layer.width, layer.height, 0, layer.opacity, layer.mode)
		svgimage.add_layer(newLayer, 0)               
		fileLayer = svgimage.layers[0]
		gimp.set_background(backParam)
		pdb.gimp_edit_fill (newLayer, FILL_BACKGROUND)
		pdb.gimp_image_lower_item_to_bottom(svgimage,newLayer)
		pdb.gimp_layer_resize_to_image_size(newLayer)
		
	if vis==1:
		for vector in svgimage.vectors:
			pdb.gimp_vectors_set_visible(vector, True)
	  
	for vector in svgimage.vectors:
		vector.name = oldname 

  # cleanup for svg does not work on Windows - Error: the file is occupied by another process
	if rem_bmp==1:
		os.remove(xchange_utf)
#		os.remove(tempsvgfilename)	
	if copy_svg==1:
		backup_tempsvgfilename = sys_file(outputFolder + os.sep + layer.name + ".svg")
		copyfile(tempsvgfilename,backup_tempsvgfilename) #back it up.	

	
	gimp.Display(svgimage)
	gimp.displays_flush()

 
register(
	"python_fu_potrace",
	"Create new image (from active layer) for tracing a bitmap, which means, transforming bitmap into a smooth, scalable image. Requires separate potrace program which must be installed in the bin Gimp directory.\nName:"+whoiam ,
	"Calls separate potrace package to trace edges.  Uses temporary files. A sequence of filters: Turdsize,Corner threshold, trace vectors, render vectors. Version:0.4",
	"MareroQ",
	"GPL",
	"2019",
	"Potrace...",
	"*",
	[
	(PF_IMAGE, "image", "Input image", None),    #current image
	(PF_DRAWABLE, "layer", "Input layer", None), #current layer	
	(PF_OPTION, "saveas", _("Save layer to a temp-file as:"),0,[" BMP ", " PPM \n(flattens the layer)"]),
	(PF_STRING, "colorParam", _("Foreground color [default black=000000]:"), "000000"),
	(PF_BOOL, "addback", _("Add background layer ? "), True),
	(PF_COLOR, "backParam", _("Background color"), (255, 255, 255)),
	(PF_SPINNER, "resolutionParam", _("Resolution in dpi:\n (dimension-based backends) "), 75, (10, 300, 1)),
	(PF_SPINNER, "turdsizeParam",_("Turdsize [default=2]:\n (suppress speckles of up to this size)"), 2, (0, 50, 0.1)),
	(PF_SPINNER, "alphamaxParam",_("Alphamax [default=1]:\n (corner threshold) "), 1, (0, 1.30, 0.05)),
	(PF_SPINNER, "opttoleranceParam",_("Opttolerance [default=0.2]:\n(curve optimization tolerance) "), .2, (0.05, 2, 0.05)),	
	(PF_OPTION, "pathParam", _("Create paths:"),0,[_("None"),_("Many, Individual"),_("One, Merged"), _("Interactive\n(for resize layer)")]),
	(PF_BOOL, "vis", _("Path visible? "), False),	
	(PF_BOOL, "rem_bmp", _("Delete temp-file?\n(file tmp/*.svg delete manually) "), False),
	(PF_BOOL, "copy_svg", _("Copy svg file to another directory?\n (as layer-name.svg) "), False),	
	(PF_DIRNAME, "outputFolder", _("Output directory to save backup svg :"), "d:\\Potrace\\"),	
	],
	[],
	python_fu_potrace,menu="<Image>/Filters/Edge-Detect/",
		domain = ("plugin-potrace", locale_directory)
	)

main()
 
